#! /usr/bin/perl
# firewall.cgi - Created by James Pattie (james@pcxperience.com)
# 07/01/2002 - PCXFirewall frontend (simple)
# Copyright (c) 2003-2005, Xperience, Inc. (http://www.pcxperience.com/)
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.

use strict;
use HTMLObject::Normal 2.25;  # We rely on the onbeforeunload support.
use HTMLObject::CGILib;
use Config::IniFiles;
use PCXFireWall::VeryTight2;
use Term::ANSIColor qw(:constants);

use lib '/etc/pcx-firewall/frontend';
# pull in the config settings
use settings;

my $version = "1.7";
my $versionString = "PCX Firewall VeryTight2 Frontend $version";

my $generatorCmd = "/usr/lib/pcx-firewall/generator";
my $generatorVersion = "2.23";  # version we require.

my $veryTight2Version = "2.6";  # version we require.
my $veryTight2Ver = $PCXFireWall::VeryTight2::VERSION;

my $prefsFile = "/etc/pcx-firewall/frontend/preferences.conf";
my %prefs = ();
my $cfgVersion = "2";
my $cfgVersionStr = "_configVersion_";  # hopefully unique [defaults] value to let me know what version the config file is.

my %defaults =
  (
    "scriptType" => "init",
    "vendorVersion" => "Debian|sarge",  # changed from RedHat 7.3 since we now use Debian exclusively.
    "backup" => "1",
    "backupNumber" => "15",
    "installScript" => "0",  # this indicates if the script should be copied into the /etc/init.d directory.
    "activateScript" => "0", # this indicates if the script should be started after generation.
  );

# parse apart the environment and get anything coming into the script.
use vars qw ( %input );
HTMLObject::CGILib::ReadParse(*input);

my $doc;
$doc = undef;  # The HTMLObject that will be created.

my $command;
$command = $input{command};
my $extraArgs;
$extraArgs = $input{extraArgs};
my $configName;
$configName = $input{configName};

if (!exists $input{command})
{
  $command = "mainMenu";
}

# make sure the pcx firewall generator command is executable.
if (! -x $generatorCmd)
{
  myDie(error => "$generatorCmd is not executable!  Error = '$!'.");
}
my $pcxfirewallVersion = "";
$pcxfirewallVersion = `$generatorCmd -v`;
$pcxfirewallVersion =~ /(\d\.\d+)/;
$pcxfirewallVersion = $1;

if ($pcxfirewallVersion < $generatorVersion)
{
  myDie(error => "$pcxfirewallVersion is not new enough!  You need PCXFirewall $generatorVersion installed.");
}
if ($veryTight2Ver < $veryTight2Version)
{
  myDie(error => "$veryTight2Ver is not new enough!  You need PCXFireWall::VeryTight2 $veryTight2Version installed.");
}

processPrefsFile($prefsFile);

if ($command eq "mainMenu")
{
  $doc = displayMainMenu();
}
elsif ($command eq "create")
{
  (my $result, $doc) = cloneConfig("template", $extraArgs);
  if ($result)
  {
    $doc = displayMainMenu(status => "$extraArgs needs to be Edited.");
  }
}
elsif ($command eq "edit")
{
  $doc = displayEditScreen(configName => $configName);
}
elsif ($command eq "delete")
{
  if ($configName =~ /^(\w+\.xml)$/)
  {
    $configName =~ s/^(\w+)(\.xml)$/$1/;
  }
  if (! -e "../configs/$configName.xml")
  {
    $doc = displayMainMenu(error => "Firewall Config name = '$configName' does not exist!  I can not delete it!");
  }
  elsif ($configName eq "template")
  {
    $doc = displayMainMenu(error => "I can not delete $configName!");
  }
  else
  {
    # remove them from the preferences section if they existed.
    if (exists $prefs{"$configName.xml"})
    {
      delete $prefs{"$configName.xml"};

      if ($prefs{defaults}{defaultConfig} eq "$configName.xml")
      {
        delete $prefs{defaults}{defaultConfig};
      }

      updatePrefsFile($prefsFile);
    }

    my $return = system("/bin/rm -f ../configs/$configName.xml");
    if ($return != 0)
    {
      $doc = displayMainMenu(error => "Deletion of $configName.xml failed!<br />Error = $!");
    }
    else
    {
      # delete the generated scripts also
      $return = system("/bin/rm -rf ../configs/VeryTight2-$configName.xml");
      if ($return != 0)
      {
        $doc = displayMainMenu(error => "Deletion of configs/VeryTight2-$configName.xml failed!<br />Error = $!");
      }
      else
      {
        $doc = displayMainMenu(status => "Deleted Firewall Config $configName.");
      }
    }
  }
}
elsif ($command eq "clone")
{
  (my $result, $doc) = cloneConfig($configName, $extraArgs);
  if ($result)
  {
    $doc = displayMainMenu(status => "$configName cloned to $extraArgs.");
  }
}
elsif ($command eq "generate")
{
  if ($configName =~ /^(\w+\.xml)$/)
  {
    $configName =~ s/^(\w+)(\.xml)$/$1/;
  }
  if (! -e "../configs/$configName.xml")
  {
    $doc = displayMainMenu(error => "Firewall Config name = '$configName' does not exist!  I can not generate it!");
  }
  else
  {
    if (exists $input{process})
    {
      # verify that we have all the necessary user input and then generate.
      my $scriptType = $input{scriptType};
      my $vendorVersion = $input{vendorVersion};
      my $backup = (exists $input{backup} ? 1 : 0);
      my $backupNumber = $input{backupNumber};
      my $installScript = (exists $input{activateScript} ? 1 : 0);
      my $activateScript = (exists $input{activateScript} ? 1 : 0);
      my $updatePrefs = (exists $input{updatePrefs} ? 1 : 0);

      my $errorStr = "";

      if (lookupScriptType($scriptType) == -1)
      {
        $errorStr .= "scriptType = '$scriptType' is invalid!<br />\n";
      }
      if ($scriptType eq "init")
      {
        if (lookupVendorVersion($vendorVersion) == -1)
        {
          $errorStr .= "vendorVersion = '$vendorVersion' is invalid!<br />\n";
        }
      }
      if ($backup && lookupBackupNumber($backupNumber) == -1)
      {
        $errorStr .= "backupNumber = '$backupNumber' is invalid!<br />\n";
      }
      if ($installScript && ($demo || !$activateScripts))
      {
        $errorStr .= "<b>Install Script</b> is illegal at this time!<br />\n";
      }
      if ($activateScript && !$installScript && ($demo || !$activateScripts))
      {
        $errorStr .= "<b>Activate Script</b> is illegal at this time!<br />\n";
      }
      if ($errorStr)
      {
        $doc = displayGenerateScreen(configName => $configName, error => $errorStr);
      }
      else
      {
        # see if the user wants the prefs updated.
        if ($updatePrefs)
        {
          # make sure the section exists.
          $prefs{"$configName.xml"} = {} if (!exists $prefs{"$configName.xml"});

          foreach my $item (qw(scriptType vendorVersion backupNumber))
          {
            $prefs{"$configName.xml"}{$item} = $input{$item} if (exists $input{$item});
          }
          foreach my $item (qw(activateScript installScript backup))
          {
            $prefs{"$configName.xml"}{$item} = (exists $input{$item} ? 1 : 0);
          }

          updatePrefsFile($prefsFile);
        }

        # see if we have to backup first
        if ($backup)
        {
          # see if there are any generated files to backup first.
          my $n = $backupNumber - 1;
          # cleanup any files we no longer should have around.
          for (my $i=50; $i > $n; $i--)
          {
            if (-f "../configs/VeryTight2-$configName.xml/firewall.$i")
            {
              $errorStr = "";
              # delete this file as we are no longer keeping this many around.
              my $return = system("/bin/rm -rf ../configs/VeryTight2-$configName.xml/firewall.$i");
              if ($return != 0)
              {
                $errorStr .= "Deletion of backup file $i for firewall $configName failed!<br />Error = $!<br />\n";
              }
            }
          }

          # now handle the n files the user wants around.
          while ($n > 0)
          {
            if (-f "../configs/VeryTight2-$configName.xml/firewall.$n")
            {
              my $return = system("/bin/mv ../configs/VeryTight2-$configName.xml/firewall.$n ../configs/VeryTight2-$configName.xml/firewall." . ($n + 1));
              if ($return != 0)
              {
                $errorStr .= "Moving of backup file $n to " . ($n + 1) . " for firewall $configName failed!<br />Error = $!<br />\n";
              }
            }
            $n--;
          }

          # now handle the final firewall case
          if (-f "../configs/VeryTight2-$configName.xml/firewall")
          {
            my $return = system("/bin/mv ../configs/VeryTight2-$configName.xml/firewall ../configs/VeryTight2-$configName.xml/firewall.1");
            if ($return != 0)
            {
              $errorStr .= "Moving of last generated file to backup file 1 for firewall $configName failed!<br />Error = $!<br />\n";
            }
          }
        }
        if ($errorStr)
        {
          $doc = displayGenerateScreen(configName => $configName, error => $errorStr);
        }
        else
        {
          # build up the command line
          my $args = "-r VeryTight2 -c $configName.xml -s $scriptType -D ../configs";
          if ($scriptType eq "init")
          {
            (my $vendor, my $version) = split (/\|/, $vendorVersion);
            $args .= " -V $vendor -d $version";
          }
          my $output = `$generatorCmd $args 2>&1`;

          $output = fixupOutput($output);

          if ($output =~ /Error|Usage/)
          {
            $doc = displayGenerateScreen(configName => $configName, error => "Cmd: $generatorCmd $args 2>&1<br />\n$output");
          }
          else
          {
            $output = "<br />" . $output;  # make sure we start out a line down on the page.

            # See if the user wanted the script made live.
            if ($installScript)
            {
              # run the install script and tell it to execute, if $activateScript = 1.
              (my $vendor, my $version) = split (/\|/, $vendorVersion);
              $args = "-s ../configs/VeryTight2-$configName.xml -t $scriptType" . ($scriptType eq "init" ? " -d $vendor" : "") . ($activateScript ? " -e" : "");
              $output = `sudo /etc/pcx-firewall/frontend/scripts/install $args`;

              $output = fixupOutput($output);

              $doc = displayMainMenu(status => $output . "<br /><br /><br />\nInstalled" . ($activateScript ? " and Activated" : "") . " Script. &nbsp;See above for results.");
            }
            else
            {
              $doc = displayMainMenu(status => $output);
            }
          }
        }
      }
    }
    else
    {
      # ask the user for the necessary info.
      $doc = displayGenerateScreen(configName => $configName);
    }
  }
}
elsif ($command eq "logout")
{
  print <<"END_OF_CODE";
Status: 401 Unauthorized
WWW-Authenticate: Basic realm="PCXFirewall Frontend"
Content-Type: text/html

<html>
  <head>
    <title>Logout</title>
  </head>
  <body>
    <a href="index.cgi">Login</a>
  </body>
</html>
END_OF_CODE
  exit 0;  # jump out early.
}
elsif ($command eq "save")
{
  # configName has the name of the xml file
  # extraArgs has the xml string to work with.
  # nextCommand tells me where to goto after I save.
  my $nextCommand = $input{nextCommand};
  my $backupFile = (exists $input{backupFile} ? 1 : 0);

  if ($configName =~ /^(\w+\.xml)$/)
  {
    $configName =~ s/^(\w+)(\.xml)$/$1/;
  }
  if (! -e "../configs/$configName.xml")
  {
    $doc = displayMainMenu(error => "Firewall Config name = '$configName' does not exist!  I can not save to it!");
  }
  elsif ($nextCommand !~ /^(mainMenu|edit|generate)$/)
  {
    $doc = displayMainMenu(error => "nextCommand = '$nextCommand' is invalid!");
  }
  else
  {
    # pull in the configuration values from the XML String - validate it.
    my $ruleObj = PCXFireWall::VeryTight2->new;
    my $confObj = PCXFireWall::ConfigParser->new;
    my $dataObj = undef;
    my %data = ();
    eval { $dataObj = $confObj->parse(string => "$extraArgs", module => "VeryTight2", moduleVersion => $ruleObj->{configFileVersion}); };
    if ($@)
    {
      $doc = displayMainMenu(error => "Error parsing XML String for config file '$configName' while trying to Save!<br />\n$@");
    }
    else
    {
      # see if they wanted us to backup the file.
      # get the current date/time value as YYYYMMDD-HHMMSS
      my $date = `/bin/date +%Y%m%d-%H%M%S`;
      chomp $date;
      if ($backupFile)
      {
        # make sure our -backup directory is created.
        if (! -d "../configs/$configName-backup")
        {
          `/bin/mkdir ../configs/$configName-backup`;
        }
        `/bin/cp ../configs/$configName.xml ../configs/$configName-backup/$date`;

        # we really should only backup the file if the new string is different.
      }

      # update the config file on disk.
      open(FILE, ">../configs/$configName.xml") or myDie(error => "Error opening config file '$configName' for saving!<br />\n$!");
      print FILE $extraArgs;
      close(FILE);

      my $message = "Successfully saved config file '$configName'.";
      if ($backupFile)
      {
        $message .= " &nbsp;Backed up file as <b>$date</b>.";
      }
      # now call the nextCommand method.
      if ($nextCommand eq "mainMenu")
      {
        $doc = displayMainMenu(status => $message);
      }
      elsif ($nextCommand eq "edit")
      {
        $doc = displayEditScreen(configName => $configName, status => $message);
      }
      elsif ($nextCommand eq "generate")
      {
        $doc = displayGenerateScreen(configName => $configName, status => $message);
      }
    }
  }
}
elsif ($command eq "manageBackups")
{
  if ($configName =~ /^(\w+\.xml)$/)
  {
    $configName =~ s/^(\w+)(\.xml)$/$1/;
  }
  if (! -d "../configs/$configName-backup")
  {
    $doc = displayMainMenu(error => "There are no backups for Firewall Config '$configName' to work with!");
  }
  else
  {
    $doc = displayManageBackupsScreen(configName => $configName);
  }
}
elsif ($command eq "deleteBackup")
{
  if ($configName =~ /^(\w+\.xml)$/)
  {
    $configName =~ s/^(\w+)(\.xml)$/$1/;
  }
  if (! -d "../configs/$configName-backup")
  {
    $doc = displayMainMenu(error => "There are no backups for Firewall Config '$configName' to delete!");
  }
  else
  {
    # get the list of delete checkboxes from the user.
    my @deleteIndexes = grep /^delete\d+$/, keys %input;
    if (scalar @deleteIndexes == 0)
    {
      $doc = displayManageBackupsScreen(configName => $configName, error => "You didn't specify any files to delete!");
    }
    else
    {
      foreach my $index (@deleteIndexes)
      {
        my $file = $input{$index};
        `/bin/rm -f ../configs/$configName-backup/$file`;
      }
      $doc = displayManageBackupsScreen(configName => $configName, status => "Deleted <b>" . (int scalar @deleteIndexes) . "</b> backup files.");
    }
  }
}
elsif ($command eq "activate")
{
  my $backupFile = $input{backupFile};

  if ($configName =~ /^(\w+\.xml)$/)
  {
    $configName =~ s/^(\w+)(\.xml)$/$1/;
  }
  if (! -d "../configs/$configName-backup")
  {
    $doc = displayMainMenu(error => "There are no backups for Firewall Config '$configName' to restore!");
  }
  else
  {
    # make sure we have the config specified in extraArgs to restore.
    if (! -e "../configs/$configName-backup/$extraArgs")
    {
      $doc = displayManageBackupsScreen(configName => $configName, error => "Config file <b>$extraArgs</b> does not exist to make active for <b>$configName</b>!");
    }
    else
    {
      my $date = `/bin/date +%Y%m%d-%H%M%S`;
      chomp $date;
      if ($backupFile)
      {
        # make sure our -backup directory is created.  (This shouldn't be necessary, but you never know.)
        if (! -d "../configs/$configName-backup")
        {
          `/bin/mkdir ../configs/$configName-backup`;
        }
        `/bin/cp ../configs/$configName.xml ../configs/$configName-backup/$date`;

        # we really should only backup the file if the new string is different.
      }

      # now we take the selected back-up file and make it the active config file.
      `/bin/cp -f ../configs/$configName-backup/$extraArgs ../configs/$configName.xml`;
      $doc = displayMainMenu(configName => $configName, status => "Successfully re-activated <b>$extraArgs</b> " . ($backupFile ? " after backing up the current config file." : "<span style=\"color: red;\"><b>but did not backup the current config file!</b></span>"));
    }
  }
}
else
{
  # display the main menu and generate an error.
  $doc = displayMainMenu(error => "Unknown command = '$command'!");
}

# set the JavaScript error handling type and e-mail address.
$doc->setJavascriptVersion("1.5");
$doc->printJavascriptRequired(version => "1.5", message => "JavaScript 1.5 is required.  You need a newer browser (Mozilla 1.7 or Firefox 1.0 tested)");
$doc->setJavascriptErrorHandlerPrgName(name => "PCX Firewall VeryTight2 Frontend");
$doc->setJavascriptErrorHandlerPrgVersion(version => "$version");
$doc->setJavascriptErrorHandlerEmail(email => "james\@pcxperience.com");
$doc->display;  # display the generated document.

exit 0;

sub cloneConfig
{
  my $doc = undef;
  my $source = shift;
  my $configName = shift;  # the name of the new config file to create from template.xml
  my $result = 1;

  # verify the config name is valid.
  if ($extraArgs =~ /^(\w+\.xml)$/)
  {
    $extraArgs =~ s/^(\w+)(\.xml)$/$1/;
  }
  # make sure it doesn't already exist.
  if (-e "../configs/$extraArgs.xml")
  {
    $doc = displayMainMenu(error => "Firewall Config name = '$extraArgs' already exists!");
    $result = 0;
  }
  else
  {
    # verify we have the source file and then copy it.
    if (! -e "../configs/$source.xml")
    {
      $doc = displayMainMenu(error => "$source does not exist!  I can not create $extraArgs without it.");
      $result = 0;
    }
    else
    {
      if ($demo)
      {
        # make sure we haven't gone over our quota
        my @files = glob("../configs/*.xml");
        my $numFiles = scalar @files;
        foreach my $file (@files)
        {
          if (-d $file)
          {
            $numFiles--;
          }
        }
        if ($numFiles + 1 > $maxDemoFiles)
        {
          $doc = displayMainMenu(error => "There are already $maxDemoFiles files created.  Sorry, I can not create $extraArgs at this time.");
          $result = 0;
        }
      }
      # only continue as long as we haven't had an issue yet.
      if ($result)
      {
        my $return = system("/bin/cp ../configs/$source.xml ../configs/$extraArgs.xml");
        if ($return != 0)
        {
          $doc = displayMainMenu(error => "Copying of $source to $extraArgs failed!<br />Error = $!");
          $result = 0;
        }
      }
    }
  }
  return ($result, $doc);
}

sub printLogoutLink
{
  my $string = "";

  if (exists $ENV{REMOTE_USER})
  {
    $string .= " | <a href=\"#\" onClick=\"triggerForm('logout'); return false;\" title=\"Logout\">Logout</a>";
  }

  return $string;
}

sub displayMainMenu
{
  my %args = ( error => "", status => "", @_ );
  my $error = $args{error};
  my $status = $args{status};
  my $doc = HTMLObject::Normal->new();

  $doc->setTitle("$versionString" . ($demo ? " (Demo)" : "") . " - Main Menu");

  # include the css file
  $doc->setLink(rel => "stylesheet", href => "../css/style.css", type => "text/css", title => "Main Style");

  # include the mainMenu.js file
  $doc->setJavascriptInclude(version => "1.5", file => "../js/mainMenu.js");

  my $configFiles = "";

  # get the list of available config files (*.xml only)
  my @files = glob("../configs/*.xml");
  my $templateExists = 0;
  my $configNameIndex = -1;
  my $counter = 0;
  foreach my $file (sort @files)
  {
    if (! -d $file)
    {
      $file =~ s/^(\.\.\/configs\/)//;  # remove the path part.
      $file =~ s/(\.xml)$//; # remove the .xml part.
      $configFiles .= "          <option value=\"$file\">$file</option>\n";

      if (exists $input{configName})
      {
        $configNameIndex = $counter if ($input{configName} eq $file);
      }
      elsif (exists $prefs{defaults}{defaultConfig})
      {
        $configNameIndex = $counter if ($prefs{defaults}{defaultConfig} eq "$file");
      }

      $templateExists = 1 if ($file eq "template");
      $counter++;
    }
  }

  $doc->setFocus("body");

  $doc->print(printError($error)) if ($error);
  $doc->print(printStatus($status)) if ($status);

  if (@files == 0)
  {
    $doc->print(printError("No config files found!"));
  }
  if (!$templateExists)
  {
    $doc->print(printError("You do not have a Template file defined!  It must be named <b>template</b>."));
  }

  my $demoString = ($demo ? " (Demo)" : "");
  my $logoutString = printLogoutLink();
  my $displayConfigsString = ($demo ? "" : " | <a href=\"../configs/\" title=\"View xml config files and generated scripts.\">View Configs</a>");

  $doc->print(<<"END_OF_CODE");
<p class="header">$versionString$demoString - Main Menu</p>
<form name="mainMenu" method="post">
  <input type="hidden" name="command" value="">
  <input type="hidden" name="extraArgs" value="">
  <table border="0" width="100%">
    <tr>
      <td width="70%" valign="top">
        <div class="border">
          <div class="content">
            <table border="0" width="100%" class="verbar">
              <tr>
                <td align="left"><span style="color: green;">Using</span>: $generatorCmd $pcxfirewallVersion</td>
                <td align="left"><span style="color: green;">VeryTight2</span>: $veryTight2Ver</td>
                <td align="right"><a href="#" onClick="triggerForm('refresh'); return false;" title="Reload the page">Refresh</a>$logoutString$displayConfigsString</td>
              </tr>
            </table>
            <br />
            <!-- Buttons, available config files, etc. -->
            <input type="button" name="create" value="Create" onClick="if (askNewName()) { triggerForm('create'); }">
            <br />
            <br />
            Available Firewall Configs:<br />
            <select name="configName" size="10" onChange="enableButtons();">
              $configFiles
            </select>
            <br /><br />
            <input type="checkbox" name="makeDefault"> Make Default Config
            <br /><br />
            <input type="button" name="editB" value="Edit" onClick="triggerForm('edit');" disabled="true">&nbsp;
            <input type="button" name="deleteB" value="Delete" onClick="triggerForm('delete');" disabled="true">&nbsp;
            <input type="button" name="cloneB" value="Clone" onClick="if (askNewName()) { triggerForm('clone'); }" disabled="true"><br />
            <input type="button" name="generateB" value="Generate" onClick="triggerForm('generate');" disabled="true">&nbsp;
            <input type="button" name="backupsB" value="Manage Backups" onClick="triggerForm('manageBackups');" disabled="true">
          </div>
        </div>
      </td>
      <td width="30%" valign="top">
        <div id="feedback" class="border">
          <div class="feedbackContent">
            The <b>Make Default Config</b> checkbox allows you to specify that you want the selected Firewall Config to be the default
            config file when you log into the interface. &nbsp;This is only updated on <b>Edit</b> and <b>Generate</b> events.<br /><br />
            Use the <b>Create</b> button to create a new Firewall Config based upon the <i>template</i> Template config file.<br /><br />
            Use the <b>Edit</b> and <b>Delete</b> buttons to edit and delete the selected config file.<br /><br />
            Use the <b>Clone</b> button to make a copy of the selected config file.<br /><br />
            Use the <b>Generate</b> button to generate the shell script for the selected config file. &nbsp;You can also make it be the currently
            active firewall rules file.<br /><br />
            Use the <b>Manage Backups</b> button to re-activate a previous version, delete or view backups of the selected config file.
          </div>
        </div>
      </td>
    </tr>
  </table>
</form>
END_OF_CODE

  $doc->setOnload(code => "document.mainMenu.configName.selectedIndex=$configNameIndex;enableButtons();");

  return $doc;
}

sub displayManageBackupsScreen
{
  my %args = ( error => "", status => "", configName => "", @_ );
  my $error = $args{error};
  my $status = $args{status};
  my $configName = $args{configName};
  my $doc = HTMLObject::Normal->new();

  my $demoString = ($demo ? " (Demo)" : "");
  $doc->setTitle("$versionString$demoString - Manage Backups of Firewall $configName");

  # include the css file
  $doc->setLink(rel => "stylesheet", href => "../css/style.css", type => "text/css", title => "Main Style");

  # include the manageBackupsScreen.js file
  $doc->setJavascriptInclude(version => "1.5", file => "../js/manageBackupsScreen.js");

  $doc->setFocus("body");

  $doc->print(printError($error)) if ($error);
  $doc->print(printStatus($status)) if ($status);

  my $logoutString = printLogoutLink();

  $doc->print(<<"END_OF_CODE");
<p class="header">$versionString$demoString - Manage Backups of Firewall <span style="color: blue;">$configName</span></p>
<form name="manageBackups" method="post">
  <input type="hidden" name="command" value="">
  <input type="hidden" name="configName" value="$configName">
  <input type="hidden" name="process" value="1">
  <input type="hidden" name="extraArgs" value="">
  <input type="hidden" name="backupFile" value="">
  <table border="0" width="100%">
    <tr>
      <td width="70%" valign="top">
        <div class="border">
          <div class="content">
            <table border="0" width="100%" class="verbar">
              <tr>
                <td align="left"><span style="color: green;">Using</span>: $generatorCmd $pcxfirewallVersion</td>
                <td align="left"><span style="color: green;">VeryTight2</span>: $veryTight2Ver</td>
                <td align="right"><a href="#" onClick="triggerForm('mainMenu'); return false;" title="Return to Main Menu">Main Menu</a>$logoutString</td>
              </tr>
            </table>
            <br />
            <!-- List available backed-up firewall config files. -->
            <table border="0" width="100%">
              <tr class="head">
                <th>&nbsp;</th><th>Delete</th><th align="left">Date / Time Stamp</th>
              </tr>
END_OF_CODE
  my @files = glob("../configs/$configName-backup/*");
  @files = sort { $b <=> $a } @files;
  my $index = 0;
  foreach my $fileName (@files)
  {
    my $rowClass = ($index % 2 == 0 ? "even" : "odd");
    (my $displayFileName = $fileName) =~ s/\.\.\/configs\/$configName-backup\///;
    $doc->print(<<"END_OF_CODE");
              <tr class="$rowClass">
                <td align="center"><input type="button" name="edit$index" value="Activate" onclick="triggerForm('activate', '$displayFileName');" /></td>
                <td align="center"><input type="checkbox" name="delete$index" value="$displayFileName" /></td>
                <td><a href="$fileName"><b>$displayFileName</b></a></td>
              </tr>
END_OF_CODE
    $index++;
  }
  $doc->print(<<"END_OF_CODE");
            </table>
            <br /><br />
            <input type="button" name="selectAll" value="Select All" onclick="selectAllToDelete();">&nbsp;
            <input type="button" name="unselectAll" value="Un-Select All" onclick="unSelectAllToDelete();">&nbsp;
            <input type="button" name="deleteB" value="Delete" onClick="triggerForm('deleteBackup');">
          </div>
        </div>
      </td>
      <td width="30%" valign="top">
        <div id="feedback" class="border">
          <div class="feedbackContent">
            Select the backed up script you want to re-edit, delete or just view.<br /><br />
            Clicking on <b>Activate</b> will allow you to make the selected config file the active config
            file. &nbsp;You will be prompted to backup <b>$configName.xml</b>. &nbsp;<b>If you do not click <i>OK</i>, then
            your currently active config file will not be backed up and will be lost.</b><br /><br />
            Clicking on the displayed file name will let you view the xml config file.<br /><br />
            Use the <b>Select All</b> button to select all files to delete.<br /><br />
            Use the <b>Un-Select All</b> button to un-select all files to delete.<br /><br />
            Use the <b>Delete</b> button to delete the files you selected.
          </div>
        </div>
      </td>
    </tr>
  </table>
</form>
END_OF_CODE

  return $doc;
}

sub displayGenerateScreen
{
  my %args = ( error => "", status => "", configName => "", @_ );
  my $error = $args{error};
  my $status = $args{status};
  my $configName = $args{configName};
  my $doc = HTMLObject::Normal->new();

  $doc->setTitle("$versionString" . ($demo ? " (Demo)" : "") . " - Generate Firewall $configName");

  # include the css file
  $doc->setLink(rel => "stylesheet", href => "../css/style.css", type => "text/css", title => "Main Style");

  # include the generateScreen.js file
  $doc->setJavascriptInclude(version => "1.5", file => "../js/generateScreen.js");

  $doc->setFocus("body");

  $doc->print(printError($error)) if ($error);
  $doc->print(printStatus($status)) if ($status);

  # user preferences (like if they want to backup their scripts and how many copies to keep) are stored
  # in the preferences file $prefsFile and are available via the %prefs hash.

  my $scriptTypeIndex;
  my $vendorVersionIndex;
  my $backupNumberIndex;
  my $backupChecked;
  my $installScriptChecked;
  my $activateScriptChecked;
  my $updatePrefsChecked;
  my $prefsConfigName = "$configName.xml";

  if (!exists $input{process})
  {
    if (exists $prefs{$prefsConfigName})
    {
      # pull defaults from the config file, if they exist.
      $scriptTypeIndex = (exists $prefs{$prefsConfigName}{scriptType} ? lookupScriptType($prefs{$prefsConfigName}{scriptType}) : lookupScriptType($defaults{scriptType}));
      $vendorVersionIndex = (exists $prefs{$prefsConfigName}{vendorVersion} ? lookupVendorVersion($prefs{$prefsConfigName}{vendorVersion}) : lookupVendorVersion($defaults{vendorVersion}));
      $backupNumberIndex = (exists $prefs{$prefsConfigName}{backupNumber} ? lookupBackupNumber($prefs{$prefsConfigName}{backupNumber}) : lookupBackupNumber($defaults{backupNumber}));
      $backupChecked = (exists $prefs{$prefsConfigName}{backup} ? $prefs{$prefsConfigName}{backup} : ($defaults{backup} ? "true" : "false"));
      $installScriptChecked = (exists $prefs{$prefsConfigName}{installScript} ? $prefs{$prefsConfigName}{installScript} : ($defaults{installScript} ? "true" : "false"));
      $activateScriptChecked = (exists $prefs{$prefsConfigName}{activateScript} ? $prefs{$prefsConfigName}{activateScript} : ($defaults{activateScript} ? "true" : "false"));
      $updatePrefsChecked = "false";
    }
    else
    {
      $scriptTypeIndex = lookupScriptType($defaults{scriptType});
      $vendorVersionIndex = lookupVendorVersion($defaults{vendorVersion});
      $backupNumberIndex = lookupBackupNumber($defaults{backupNumber});
      $backupChecked = ($defaults{backup} ? "true" : "false");
      $installScriptChecked = ($defaults{installScript} ? "true" : "false");
      $activateScriptChecked = ($defaults{activateScript} ? "true" : "false");
      $updatePrefsChecked = "true";
    }
    if (exists $input{makeDefault})
    {
      $prefs{defaults}{defaultConfig} = $prefsConfigName;
      updatePrefsFile($prefsFile);
    }
  }
  else
  {
    $scriptTypeIndex = (exists $input{scriptType} ? lookupScriptType($input{scriptType}) : lookupScriptType($defaults{scriptType}));
    $vendorVersionIndex = (exists $input{vendorVersion} ? lookupVendorVersion($input{vendorVersion}) : lookupVendorVersion($defaults{vendorVersion}));
    $backupNumberIndex = (exists $input{backupNumber} ? lookupBackupNumber($input{backupNumber}) : lookupBackupNumber($defaults{backupNumber}));
    $backupChecked = (exists $input{backup} ? "true" : ($defaults{backup} ? "true" : "false"));
    $installScriptChecked = (exists $input{installScript} ? "true" : ($defaults{installScript} ? "true" : "false"));
    $activateScriptChecked = (exists $input{activateScript} ? "true" : ($defaults{activateScript} ? "true" : "false"));
    $updatePrefsChecked = (exists $input{updatePrefs} ? "true" : "false");
  }

  my $demoString = ($demo ? " (Demo)" : "");
  my $logoutString = printLogoutLink();

  $doc->print(<<"END_OF_CODE");
<p class="header">$versionString$demoString - Generate Firewall <span style="color: blue;">$configName</span></p>
<form name="generate" method="post">
  <input type="hidden" name="command" value="">
  <input type="hidden" name="configName" value="$configName">
  <input type="hidden" name="process" value="1">
  <input type="hidden" name="extraArgs" value="">
  <table border="0" width="100%">
    <tr>
      <td width="70%" valign="top">
        <div class="border">
          <div class="content">
            <table border="0" width="100%" class="verbar">
              <tr>
                <td align="left"><span style="color: green;">Using</span>: $generatorCmd $pcxfirewallVersion</td>
                <td align="left"><span style="color: green;">VeryTight2</span>: $veryTight2Ver</td>
                <td align="right"><a href="#" onClick="triggerForm('mainMenu'); return false;" title="Return to Main Menu">Main Menu</a>$logoutString</td>
              </tr>
            </table>
            <br />
            <!-- Generate Firewall options. -->
            Script Type:<br />
            <select name="scriptType" onChange="toggleVendorVersion();">
              <option value="init" checked>Init Script</option>
              <option value="normal">Normal Shell Script</option>
            </select>
            <br /><br />
            Linux Vendor and Distro Version:<br />
            <select name="vendorVersion" disabled="true">
              <option value="RedHat|7.0">RedHat 7.0</option>
              <option value="RedHat|7.1">RedHat 7.1</option>
              <option value="RedHat|7.2">RedHat 7.2</option>
              <option value="RedHat|7.3">RedHat 7.3</option>
              <option value="RedHat|8.0">RedHat 8.0</option>
              <option value="RedHat|9">RedHat 9</option>
              <option value="RedHat|FC1">RedHat FC1</option>
              <option value="RedHat|FC2">RedHat FC2</option>
              <option value="RedHat|FC3">RedHat FC3</option>
              <option value="Debian|woody">Debian woody</option>
              <option value="Debian|sarge">Debian sarge</option>
              <option value="Debian|testing">Debian testing</option>
              <option value="Debian|unstable">Debian unstable</option>
            </select>
            <br /><br />
            <input type="checkbox" name="backup" onClick="toggleBackupNumber();"> Backup generated script
            <br /><br />
            Number of Backups to keep:<br />
            <select name="backupNumber">
              <option value="5">5</option>
END_OF_CODE
  for (my $num=10; $num <= 50; $num+=5)
  {
    if ((!$demo) || ($demo && $maxDemoFiles >= $num))
    {
      $doc->print(<<"END_OF_CODE");
              <option value="$num">$num</option>
END_OF_CODE
    }
  }
  $doc->print(<<"END_OF_CODE");
            </select>
            <br /><br />
END_OF_CODE

  if (!$demo && $activateScripts)
  {
    $doc->print(<<"END_OF_CODE");
            <input type="checkbox" name="installScript" onClick="verifyInstallScript();"> Install Script
            <br />Install the generated script into /etc/init.d or /etc/pcx-firewall (requires <b>Script Type</b> = <b>Init Script</b> selected) <b>(dangerous!)</b>
            <br /><br />
            <input type="checkbox" name="activateScript" onClick="verifyActivateScript();"> Activate Script
            <br />Make the generated script be the default Firewall Rules file <b>(dangerous!)</b>
            <br />You must select <b>Install Script</b> to be able to <b>Activate</b>.
            <br /><br />
END_OF_CODE
  }

  $doc->print(<<"END_OF_CODE");
            <input type="checkbox" name="updatePrefs"> Update Preferences
            <br /><br />
            <input type="button" name="generateB" value="Generate" onClick="triggerForm('generate');">
          </div>
        </div>
      </td>
      <td width="30%" valign="top">
        <div id="feedback" class="border">
          <div class="feedbackContent">
            Specify the <b>Script Type</b> you want generated.<br /><br />
            - <b>Init Script</b> will generate a script that is capable of being
            dropped into /etc/init.d and can have <b>chkconfig</b> run on it under RedHat.
            <br /><br />
            - <b>Normal Shell Script</b> will generate a script that will function like the Init Script, but won't support <b>chkconfig</b>.<br /><br />
            If generating an <b>Init Script</b> then you also need to select the Linux Distro and Version you are generating for.<br /><br />
            If you check the <b>Backup generated script</b> checkbox, then any previously generated firewall scripts for <i>$configName</i>
            will be backed up.  Use the <b>Number of Backups to keep</b> dropdown to determine how many scripts to keep backed up. &nbsp;<i>This number is how many copies to keep, not how many total firewall files will be in the VeryTight2 directory.</i><br /><br />
            By selecting the <b>Install Script</b> checkbox, the generated script will be copied into your /etc/init.d directory (if Script Type = Init Script) or into your /etc/pcx-firewall directory (if Script Type = Normal Shell Script), thus making it available to be run at system startup. <i>(Not available in Demo mode.)</i><br /><br />
            By selecting the <b>Activate Script</b> checkbox, the generated script will be made your currently active firewall rules. &nbsp;This requires <b>Install Script</b> to be selected. <i>(Not available in Demo mode.)</i><br /><br />
            By selecting the <b>Update Preferences</b> checkbox, you will remember the current settings for this firewall configuration
            thus making re-generation in the future easier.<br /><br />
            Click the <b>Generate</b> button to actually run the XML config file through the VeryTight2 Rule module and generate the
            shell script that represents the firewall scenario you built.
          </div>
        </div>
      </td>
    </tr>
  </table>
</form>
END_OF_CODE

  my $code = "document.generate.scriptType.selectedIndex=$scriptTypeIndex;\n";
  $code .= "document.generate.vendorVersion.selectedIndex=$vendorVersionIndex;\n";
  $code .= "document.generate.backupNumber.selectedIndex=$backupNumberIndex;\n";
  $code .= "document.generate.backup.checked=$backupChecked;\n";
  $code .= (!$demo && $activateScripts ? "document.generate.activateScript.checked=$activateScriptChecked;\ndocument.generate.installScript.checked=$installScriptChecked;\nverifyInstallScript();\nverifyActivateScript();\n" : "");
  $code .= "document.generate.updatePrefs.checked=$updatePrefsChecked;\n";
  $code .= "toggleVendorVersion();\n";
  $code .= "toggleBackupNumber();\n";

  $doc->setOnload(code => $code);

  return $doc;
}

sub displayEditScreen
{
  my %args = ( error => "", status => "", configName => "", @_ );
  my $error = $args{error};
  my $status = $args{status};
  my $configName = $args{configName};
  my $doc = HTMLObject::Normal->new();

  $doc->setTitle("$versionString" . ($demo ? " (Demo)" : "") . " - Edit Firewall $configName");

  # include the css file
  $doc->setLink(rel => "stylesheet", href => "../css/style.css", type => "text/css", title => "Main Style");

  # include the editScreen.js file
  $doc->setJavascriptInclude(version => "1.5", file => "../js/editScreen.js");

  # include the JavaScript files for each edit feature (config, zones, networks, reserved, services, paths)
  $doc->setJavascriptInclude(version => "1.5", file => "../js/editConfig.js");
  $doc->setJavascriptInclude(version => "1.5", file => "../js/editNetworks.js");
  $doc->setJavascriptInclude(version => "1.5", file => "../js/editReserved.js");
  $doc->setJavascriptInclude(version => "1.5", file => "../js/editZones.js");
  $doc->setJavascriptInclude(version => "1.5", file => "../js/editServices.js");
  $doc->setJavascriptInclude(version => "1.5", file => "../js/editPaths.js");

  # parse the specified config file and convert into JavaScript data structures.
  if (! -e "../configs/$configName.xml")
  {
    return displayMainMenu(error => "config '$configName' does not exist!");
  }

  # pull in the configuration values from the XML File.
  my $ruleObj = PCXFireWall::VeryTight2->new;
  my $confObj = PCXFireWall::ConfigParser->new;
  my $dataObj = undef;
  my %data = ();
  eval { $dataObj = $confObj->parse(file => "../configs/$configName.xml", module => "VeryTight2", moduleVersion => $ruleObj->{configFileVersion}); };
  if ($@)
  {
    return displayMainMenu(error => "Error parsing XML Config File '$configName'!<br />\n" . fixupOutput($@));
  }
  %data = %{$dataObj->{data}};

  my $servicesObj = undef;
  my %services = ();
  if ($configName ne "template")
  {
    # parse the template.xml config file and make it's Services available for importing.
    eval { $servicesObj = $confObj->parse(file => "../configs/template.xml", module => "VeryTight2", moduleVersion => $ruleObj->{configFileVersion}); };
    if ($@)
    {
      return displayMainMenu(error => "Error parsing XML Config File 'template' for Service Import Feature!<br />\n" . fixupOutput($@));
    }
    %services = %{$servicesObj->{data}};
  }

  # now walk the returned data object and create in JavaScript.
  $doc->setFocus("javascript");

  $doc->print(<<"END_OF_CODE");

function resetConfig()
{
  dataObj = new Object;      // The main object that conglomerates all the data together.
  var infoObj = new Object;  // Stores the _info_ stuff
  var configObj = new Object;// Stores the <config> info
  var networks = new Array;  // Stores the <networks> info
  var reserved = new Array;  // Stores the <reserved> info
  var zones = new Array;     // Stores the <zones> info
  var services = new Array;  // Stores the <services> info
  var paths = new Array;     // Stores the <paths> info

  // build up the info object
  infoObj.module = "$data{'_info_'}{module}";
  infoObj.version = "$data{'_info_'}{version}";
  infoObj.confFile = "$data{'_info_'}{confFile}";

  // build up the configObj
  configObj.networkCommand = "$data{config}{networkCommand}";

  configObj.zones = new Array;
  configObj.zones['internal'] = $data{config}{zones}{internal};
  configObj.zones['ipsec'] = $data{config}{zones}{ipsec};
  configObj.zones['dmz'] = $data{config}{zones}{dmz};
  configObj.zones['dialin'] = $data{config}{zones}{dialin};

  configObj.mangleRules = $data{config}{mangleRules};
  configObj.ecn = $data{config}{ecn};

  configObj.bridgeSupport = $data{config}{bridgeSupport};
  configObj.snortInlineSupport = $data{config}{snortInlineSupport};
  configObj.dynamicInterfacesMode = "$data{config}{dynamicInterfacesMode}";

  configObj.validityChecks = new Array;
  configObj.validityChecks['unclean'] = $data{config}{validityChecks}{unclean};
  configObj.validityChecks['tcpFlags'] = $data{config}{validityChecks}{tcpFlags};

  configObj.moduleSupport = new Object;
  configObj.moduleSupport.modular = $data{config}{moduleSupport}{modular};
  configObj.moduleSupport.modules = new Array;
  configObj.moduleSupport.conntrackers = new Array;
END_OF_CODE

  foreach my $module (keys %{$data{config}{moduleSupport}{conntrackers}})
  {
    $doc->print(<<"END_OF_CODE");

  configObj.moduleSupport.conntrackers['$module'] = new createConntrackerEntry($data{config}{moduleSupport}{conntrackers}{$module}{active}, "$data{config}{moduleSupport}{conntrackers}{$module}{params}");
END_OF_CODE
  }

  foreach my $module (@{$data{config}{moduleSupport}{modules}})
  {
    $doc->print(<<"END_OF_CODE");

  configObj.moduleSupport.modules.push('$module');
END_OF_CODE
  }

  $doc->print(<<"END_OF_CODE");
  configObj.logInfo = new Array;
  configObj.logInfo['prefix'] = "$data{config}{logInfo}{prefix}";
  configObj.logInfo['level'] = "$data{config}{logInfo}{level}";

  configObj.limit = new Array;
  configObj.limit['log'] = new Object;
  configObj.limit['log'].taint = $data{config}{limit}{log}{taint};
  configObj.limit['log'].reserved = $data{config}{limit}{log}{reserved};
  configObj.limit['log'].policy = $data{config}{limit}{log}{policy};
  configObj.limit['log'].reject = $data{config}{limit}{log}{reject};
  configObj.limit['log'].normal = $data{config}{limit}{log}{normal};

  configObj.limit['icmp'] = $data{config}{limit}{icmp};

  configObj.limitRates = new Array;
  configObj.limitRates['log'] = new Object;
  configObj.limitRates['log'].taint = "$data{config}{limitRates}{log}{taint}";
  configObj.limitRates['log'].reserved = "$data{config}{limitRates}{log}{reserved}";
  configObj.limitRates['log'].policy = "$data{config}{limitRates}{log}{policy}";
  configObj.limitRates['log'].reject = "$data{config}{limitRates}{log}{reject}";
  configObj.limitRates['log'].normal = "$data{config}{limitRates}{log}{normal}";

  configObj.limitRates['icmp'] = "$data{config}{limitRates}{icmp}";

  // build up the networks array
END_OF_CODE

  foreach my $index (sort { $a <=> $b } keys %{$data{config}{networks}})
  {
    $doc->print(<<"END_OF_CODE");
  networks[$index] = new createNetworkEntry("$data{config}{networks}{$index}{host}", "$data{config}{networks}{$index}{type}", $data{config}{networks}{$index}{used}, $data{config}{networks}{$index}{active}, "$data{config}{networks}{$index}{comment}");
END_OF_CODE
  }

  $doc->print(<<"END_OF_CODE");

  // build up the reserved array
END_OF_CODE

  foreach my $reserved (@{$data{config}{reserved}})
  {
    $doc->print("  reserved.push(new createReservedEntry(\"$reserved->{host}\", ");
    $doc->print(outputJSArray($reserved->{limitTo}));
    $doc->print(", $reserved->{active}, \"$reserved->{comment}\"));\n");
  }

  $doc->print(<<"END_OF_CODE");

  // build up the zones
  zones['aliasLookup'] = new Array;
END_OF_CODE

  foreach my $alias (keys %{$data{config}{interfaces}{aliasLookup}})
  {
    $doc->print("  zones['aliasLookup']['$alias'] = \"$data{config}{interfaces}{aliasLookup}{$alias}\";\n");
  }

  foreach my $zone (qw(external internal ipsec dmz dialin))
  {
    $doc->print("  zones['$zone'] = new Array;\n");
    if (scalar keys %{$data{config}{interfaces}{$zone}} == 0)
    {
      next if ($zone !~ /^(external)$/ && $data{config}{zones}{$zone} == 0);  # not being used.
    }
    else
    {
      foreach my $alias (sort keys %{$data{config}{interfaces}{$zone}})
      {
        my $info = $data{config}{interfaces}{$zone}{$alias};
        if ($zone =~ /^(ipsec)$/)
        {
          $doc->print("  zones['$zone']['$alias'] = new createIPSecZoneEntry(\"$info->{name}\"");
        }
        else
        {
          $doc->print("  zones['$zone']['$alias'] = new createOtherZoneEntry(\"$info->{name}\"");
        }
        if ($zone =~ /^(external|internal|dmz|dialin)$/)
        {
          $doc->print(", \"$info->{ip}\", $info->{ipsec}");
          $doc->print(", " . outputJSArray($info->{ipsecLimitFrom}));
        }
        else
        {
          $doc->print(", \"$info->{boundTo}\"");
        }

        $doc->print(", " . outputJSArray($info->{network}));

        $doc->print(", $info->{proxyarp}, $info->{active}, \"$info->{comment}\");\n");
      }
    }
  }

  $doc->print(<<"END_OF_CODE");

  // build up the services
END_OF_CODE

  $doc->print("  services['servicesOrder'] = " . outputJSArray($data{services}{servicesOrder}) . ";\n");

  for (my $count=0; $count < scalar(@{$data{services}{servicesOrder}}); $count++)
  {
    my $service = $data{services}{servicesOrder}[$count];
    $doc->print(<<"END_OF_CODE");

  services['$service'] = new Object;
  services['$service'].comment = "$data{services}{$service}{comment}";
  services['$service'].infos = new Array;
END_OF_CODE
    for (my $counter=0; $counter < scalar(@{$data{services}{$service}{infos}}); $counter++)
    {
      my $info = $data{services}{$service}{infos}[$counter];
      $doc->print(<<"END_OF_CODE");
  services['$service'].infos.push(new createServiceInfoEntry("$info->{proto}", "$info->{dport}", "$info->{sport}", "$info->{icmpType}"));
END_OF_CODE
    }
  }

  # define the masterServices array regardless of which file being edited so it can
  # serve as an undo feature when deleting a Service entry.
  $doc->print(<<"END_OF_CODE");


  // build up the masterServices array
  var masterServices = new Array;  // Stores the <services> info available for import.
  masterServices['servicesOrder'] = new Array;
END_OF_CODE

  if ($configName ne "template")
  {
    for (my $count=0; $count < scalar(@{$services{services}{servicesOrder}}); $count++)
    {
      my $service = $services{services}{servicesOrder}[$count];

      # for now, I'm just going to implement the simple case of listing those Service entries that don't exist in the
      # firewall config being edited.
      next if (exists $data{services}{$service});

      $doc->print(<<"END_OF_CODE");

  masterServices['servicesOrder'].push("$service");  // update the servicesOrder array.
  masterServices['$service'] = new Object;
  masterServices['$service'].comment = "$services{services}{$service}{comment}";
  masterServices['$service'].infos = new Array;
END_OF_CODE
      for (my $counter=0; $counter < scalar(@{$services{services}{$service}{infos}}); $counter++)
      {
        my $info = $services{services}{$service}{infos}[$counter];
        $doc->print(<<"END_OF_CODE");
  masterServices['$service'].infos.push(new createServiceInfoEntry("$info->{proto}", "$info->{dport}", "$info->{sport}", "$info->{icmpType}"));
END_OF_CODE
      }
    }
  }

  # now add to the dataObj.
  $doc->print("\n  dataObj.masterServices = masterServices;\n");

  $doc->print(<<"END_OF_CODE");

  // build up the paths
END_OF_CODE

  $doc->print("  paths['serviceGroupsOrder'] = " . outputJSArray($data{paths}{serviceGroupsOrder}) . ";\n");

  for (my $count=0; $count < scalar(@{$data{paths}{serviceGroupsOrder}}); $count++)
  {
    my $serviceGroup = $data{paths}{serviceGroupsOrder}[$count];
    $doc->print(<<"END_OF_CODE");

  paths['$serviceGroup'] = new Array;
  paths['$serviceGroup']['comment'] = "$data{paths}{$serviceGroup}{comment}";
  paths['$serviceGroup']['active'] = $data{paths}{$serviceGroup}{active};
END_OF_CODE

    $doc->print("  paths['$serviceGroup']['zonePathOrder'] = " . outputJSArray($data{paths}{$serviceGroup}{zonePathOrder}) . ";\n");

    foreach my $path (@{$data{paths}{$serviceGroup}{zonePathOrder}})
    {
      $doc->print(<<"END_OF_CODE");
  paths['$serviceGroup']['$path'] = new Array;
END_OF_CODE
      for (my $counter=0; $counter < scalar(@{$data{paths}{$serviceGroup}{$path}}); $counter++)
      {
        my $info = $data{paths}{$serviceGroup}{$path}[$counter];
        $doc->print("  paths['$serviceGroup']['$path'][$counter] = new createServiceGroupInfoEntry(" . (exists $info->{i} ? outputJSArray($info->{i}) : "[]"));
        $doc->print(", \"$info->{ip}\", " . (exists $info->{o} ? outputJSArray($info->{o}) : "[]"));
        if (exists $info->{limitFrom})
        {
          $doc->print(", " . outputJSArray($info->{limitFrom}));
        }
        else
        {
          $doc->print(", []");
        }
        if (exists $info->{limitTo})
        {
          $doc->print(", " . outputJSArray($info->{limitTo}));
        }
        else
        {
          $doc->print(", []");
        }
        $doc->print(", $info->{reject}, $info->{dropNoLog}, $info->{dnat}, \"$info->{dnatDestination}\"");
        $doc->print(", \"$info->{dnatZone}\", \"$info->{dnatInterface}\", \"$info->{dnatPort}\"");
        $doc->print(", $info->{redirect}, \"$info->{redirectPort}\", $info->{queue}, $info->{log}");
        $doc->print(", \"$info->{logComment}\", \"$info->{logLevel}\", " . outputJSArray($info->{service}));
        $doc->print(", $info->{active}, \"$info->{comment}\");\n");
      }
    }
  }

  $doc->print(<<"END_OF_CODE");

  // put it all together
  dataObj.info     = infoObj;
  dataObj.config   = configObj;
  dataObj.networks = networks;
  dataObj.reserved = reserved;
  dataObj.zones    = zones;
  dataObj.services = services;
  dataObj.paths    = paths;
}
END_OF_CODE

  $doc->setFocus("body");

  $doc->print(printError($error)) if ($error);
  $doc->print(printStatus($status)) if ($status);

  # user preferences (like if they want to backup their scripts and how many copies to keep) are stored
  # in the preferences file $prefsFile and are available via the %prefs hash.

  if (exists $input{makeDefault})
  {
    $prefs{defaults}{defaultConfig} = $configName;
    updatePrefsFile($prefsFile);
  }

  my $demoString = ($demo ? " (Demo)" : "");
  my $logoutString = printLogoutLink();

  $doc->print(<<"END_OF_CODE");
<p class="header">$versionString$demoString - Edit Firewall <span style="color: blue;">$configName</span></p>
<form name="editF" method="post">
  <input type="hidden" name="command" value="save">
  <input type="hidden" name="configName" value="$configName">
  <input type="hidden" name="extraArgs" value="">
  <div class="border">
    <div class="content">
      <table border="0" width="100%" class="verbar">
        <tr>
          <td align="left"><span style="color: green;">Using</span>: $generatorCmd $pcxfirewallVersion</td>
          <td align="left"><span style="color: green;">VeryTight2</span>: $veryTight2Ver</td>
          <td align="right"><a href="#" onClick="triggerForm('mainMenu'); return false;" title="Return to Main Menu">Main Menu</a> | <a href="#" onClick="processReset();return false;" title="Reset Data Structure">Reset</a>$logoutString</td>
        </tr>
      </table>
      <div id="menubar" class="contentMenu">
        <table border="0" width="100%">
          <tr>
            <td align="left" valign="top">
              <a href="#" onClick="redisplayEditScreen(); return false;" title="Edit Screen Main Page">Main</a>
              &nbsp;| <a href="#" onClick="displayConfig(); return false;" title="Edit &lt;config&gt; section">Config</a>
              &nbsp;| <a href="#" onClick="displayNetworks(); return false;" title="Edit &lt;networks&gt; section">Networks</a>
              &nbsp;| <a href="#" onClick="displayReserved(); return false;" title="Edit &lt;reserved&gt; section">Reserved</a>
              &nbsp;| <a href="#" onClick="displayZones(); return false;" title="Edit &lt;zones&gt; section">Zones</a>
              &nbsp;| <a href="#" onClick="displayServices(); return false;" title="Edit &lt;services&gt; section">Services</a>
              &nbsp;| <a href="#" onClick="displayServiceGroups(); return false;" title="Edit &lt;paths&gt; section">Paths</a>
              &nbsp;| <a href="#" onClick="displayXML(); return false;">Display XML</a>
            </td>
            <td align="right">
              <input type="checkbox" name="backupFile" value="1" checked="1"> Backup
              <input type="radio" name="nextCommand" value="mainMenu"> Save
              <input type="radio" name="nextCommand" value="edit" checked="true"> Re-Edit
              <input type="radio" name="nextCommand" value="generate"> Generate &nbsp;
              <input type="submit" name="saveB" value="Save" onClick="return saveConfig();">
            </td>
          </tr>
        </table>
      </div>
    </div>
  </div>
</form>
<div class="border">
  <div class="content">
    <div id="status">
    <b>Status</b>:
    </div>
  </div>
</div>
<br />
<div class="border">
  <div class="content">
    <form name="configF">
    <div id="content">
    </div>
    </form>
  </div>
</div>
END_OF_CODE

  my $code = "resetConfig();redisplayEditScreen();";

  $doc->setOnload(code => $code);
  $doc->onbeforeunload("return verifyChangesSavedWindowClosing();");

  return $doc;
}

sub lookupScriptType
{
  my $scriptType = shift;
  my %scriptTypeLookup =
     (
        "init"   => 0,
        "normal" => 1,
     );

  return (exists $scriptTypeLookup{$scriptType} ? $scriptTypeLookup{$scriptType} : -1);
}

sub lookupVendorVersion
{
  my $vendorVersion = shift;
  my %vendorVersionLookup =
     (
        "RedHat|7.0" => 0,
        "RedHat|7.1" => 1,
        "RedHat|7.2" => 2,
        "RedHat|7.3" => 3,
        "RedHat|8.0" => 4,
        "RedHat|9" => 5,
        "RedHat|FC1" => 6,
        "RedHat|FC2" => 7,
        "RedHat|FC3" => 8,
        "Debian|woody" => 9,
        "Debian|sarge" => 10,
        "Debian|testing" => 11,
        "Debian|unstable" => 12,
     );

  return (exists $vendorVersionLookup{$vendorVersion} ? $vendorVersionLookup{$vendorVersion} : -1);
}

sub lookupBackupNumber
{
  my $backupNumber = shift;
  my %backupNumberLookup =
     (
        "5"  => 0,
     );
  my $counter = 1;
  foreach my $num (10, 15, 20, 25, 30, 35, 40, 45, 50)
  {
    if ((!$demo) || ($demo && $maxDemoFiles >= $num))
    {
      $backupNumberLookup{$num} = $counter++;
    }
  }

  return (exists $backupNumberLookup{$backupNumber} ? $backupNumberLookup{$backupNumber} : (!$demo ? -1 : 0));
}

sub printError
{
  my $msg = shift;
  my $result = "<p><span class=\"error\">Error: </span>$msg</p>\n";
  return $result;
}

sub printStatus
{
  my $msg = shift;
  my $result = "<p><span class=\"status\">Status: </span>$msg</p>\n";
  return $result;
}

# either open or create the config file.  Make sure our defaults section exists and
# the file is uptodate.
sub processPrefsFile
{
  my $file = shift;
  my $saveFile = 0;
  my $profile = "defaults";

  # check for the config file
  if (-f $file)
  {
    # tie to it.
    tie %prefs, 'Config::IniFiles', (-file => $file) or myDie(error => "Opening config file '$file' failed! $!");

    if (exists $prefs{$profile})
    {
      # check version of the config file.
      if ($prefs{$profile}{$cfgVersionStr} != $cfgVersion)
      {
        # we need to upgrade
        if ($prefs{$profile}{$cfgVersionStr} == 1)
        {
          #print "Updating config file to version 2...\n";
          # add the items / actions that need to be done for version 1 -> 2.
          $prefs{$profile}{$cfgVersionStr} = 2;
          $prefs{$profile}{installScript} = $defaults{installScript};

          # now I need to loop over all other profiles and add the installScript option.
          foreach my $key (keys %prefs)
          {
            next if ($key eq $profile);  # skip the default profile.
            if ($prefs{$key}{activateScript})
            {
              $prefs{$key}{installScript} = 1;  # they were doing this before, make sure it still works.
            }
            else
            {
              $prefs{$key}{installScript} = $defaults{installScript};
            }
          }

          $saveFile = 1;
        }
      }

      # update the defaults stored.
      foreach my $arg (keys %defaults)
      {
        if (exists $prefs{$profile}{$arg})
        {
          $defaults{$arg} = $prefs{$profile}{$arg};
          #print "settings{$arg} = '" . $settings{$arg} . "'\n";
        }
      }
    }
    else
    {
      # create the [defaults] profile.
      $prefs{$profile} = {};  # make sure the section exists.

      foreach my $arg (keys %defaults)
      {
        $prefs{$profile}{$arg} = $defaults{$arg};
        #print "prefs{$profile}{$arg} = '" . $settings{$arg} . "'\n" if $settings{Debug};
      }

      $saveFile = 1;
    }
  }
  else  # create the config file
  {
    # we have to create the config file and tie to it.
    tie %prefs, 'Config::IniFiles', () or myDie(error => "Initializing config file '$file' failed! $!");

    # now set the name to work with.
    tied(%prefs)->SetFileName($file) or myDie(error => "Setting config file to '$file' failed! $!");

    $prefs{$profile} = {};  # make sure the section exists.

    foreach my $arg (keys %defaults)
    {
      $prefs{$profile}{$arg} = $defaults{$arg};
      #print "prefs{$profile}{$arg} = '" . $settings{$arg} . "'\n" if $settings{Debug};
    }

    $saveFile = 1;
  }

  if ($saveFile)
  {
    # set the config file version
    $prefs{$profile}{$cfgVersionStr} = $cfgVersion;

    updatePrefsFile($file);
  }
}

sub updatePrefsFile
{
  my $file = shift;

  # now save the updated config file before we continue.
  tied(%prefs)->RewriteConfig or myDie(error => "Writing config file '$file' failed!  $!");
}

# string outputJSArray(array ref)
# outputs [ and ] and the quoted, comma delimited contents of the array passed in by reference.
sub outputJSArray
{
  my $array = shift;
  my $output = "";

  $output = "[";
  $output .= "\"" if (@{$array});
  $output .= join("\",\"", @{$array});
  $output .= "\"" if (@{$array});
  $output .= "]";

  return $output;
}

# myDie - Takes lang, encoding, error
sub myDie
{
  my %args = ( lang => 'en', encoding => 'iso-8859-1', error => "", @_ );
  my $lang = $args{lang};
  my $encoding = $args{encoding};
  my $message = $args{error};
  my $dateStamp = `/bin/date`;
  my $doc = undef;

  chomp $dateStamp;

  $doc = HTMLObject::Base->new;
  $doc->setLanguageEncoding(language => $lang, encoding => $encoding);
  $doc->setTitle("PCXFirewall Frontend - Error!");
  $doc->setFocus("body");
  $doc->print("<center><h1>Error Occurred!</h1></center>\n");
  $doc->print("<br /><br />\n");
  $doc->print($message);
  $doc->print("<br />We came from '$ENV{HTTP_REFERER}'.<br />\n") if (exists $ENV{HTTP_REFERER});

  $doc->display();  # only display the error document once, either the actual error or the error plus a logging error.

  exit 1;
}

sub fixupOutput
{
  my $input = shift;
  my $GREEN = GREEN;
  my $RESET = RESET;
  my $RED   = RED;
  my $BLUE  = BLUE;

  # fixup the colors (special characters, etc.)
  $GREEN =~ s/(\[|\/|\<)/\\$1/g;
  $RESET =~ s/(\[|\/|\<)/\\$1/g;
  $RED =~ s/(\[|\/|\<)/\\$1/g;
  $BLUE =~ s/(\[|\/|\<)/\\$1/g;

  $input =~ s/&/&amp;/g;
  $input =~ s/  /&nbsp; /g;
  $input =~ s/</&lt;/g;
  $input =~ s/>/&gt;/g;
  $input =~ s/\n/<br \/>\n/g;

  $input =~ s/$GREEN/\<span style="color: green;"\>/g;
  $input =~ s/$RESET/\<\/span\>/g;
  $input =~ s/$RED/\<span style="color: red;"\>/g;
  $input =~ s/$BLUE/\<span style="color: blue;"\>/g;

  return $input;
}
